@@ -61,6 +61,10 @@ pub enum Bson {
     Int32(i32),
     /// 64-bit signed integer
     Int64(i64),
+    /// 32-bit signed integer
+    UInt32(u32),
+    /// 64-bit signed integer
+    UInt64(u64),
     /// Timestamp
     Timestamp(Timestamp),
     /// Binary data
@@ -122,6 +126,8 @@ impl Display for Bson {
             }
             Bson::Int32(i) => write!(fmt, "{}", i),
             Bson::Int64(i) => write!(fmt, "{}", i),
+            Bson::UInt32(i) => write!(fmt, "{}", i),
+            Bson::UInt64(i) => write!(fmt, "{}", i),
             Bson::Timestamp(ref x) => write!(fmt, "{}", x),
             Bson::Binary(ref x) => write!(fmt, "{}", x),
             Bson::ObjectId(ref id) => write!(fmt, "ObjectId(\"{}\")", id),
@@ -159,6 +165,8 @@ impl Debug for Bson {
             Bson::JavaScriptCodeWithScope(ref code) => Debug::fmt(code, fmt),
             Bson::Int32(i) => fmt.debug_tuple("Int32").field(&i).finish(),
             Bson::Int64(i) => fmt.debug_tuple("Int64").field(&i).finish(),
+            Bson::UInt32(i) => fmt.debug_tuple("UInt32").field(&i).finish(),
+            Bson::UInt64(i) => fmt.debug_tuple("UInt64").field(&i).finish(),
             Bson::Timestamp(ref t) => Debug::fmt(t, fmt),
             Bson::Binary(ref b) => Debug::fmt(b, fmt),
             Bson::ObjectId(ref id) => Debug::fmt(id, fmt),
@@ -290,14 +298,17 @@ impl From<i64> for Bson {
 
 impl From<u32> for Bson {
     fn from(a: u32) -> Bson {
-        if let Ok(i) = i32::try_from(a) {
-            Bson::Int32(i)
-        } else {
-            Bson::Int64(a.into())
-        }
+        Bson::UInt32(a)
     }
 }
 
+impl From<u64> for Bson {
+    fn from(a: u64) -> Bson {
+        Bson::UInt64(a)
+    }
+}
+
+
 impl From<[u8; 12]> for Bson {
     fn from(a: [u8; 12]) -> Bson {
         Bson::ObjectId(oid::ObjectId::from_bytes(a))
@@ -379,12 +390,10 @@ impl Bson {
             }
             Bson::Double(v) => json!(v),
             Bson::String(v) => json!(v),
-            Bson::Array(v) => Value::Array(v.into_iter().map(Bson::into_relaxed_extjson).collect()),
-            Bson::Document(v) => Value::Object(
-                v.into_iter()
-                    .map(|(k, v)| (k, v.into_relaxed_extjson()))
-                    .collect(),
-            ),
+            Bson::Array(v) => json!(v),
+            Bson::Document(v) => {
+                Value::Object(v.into_iter().map(|(k, v)| (k, Value::from(v))).collect())
+            }
             Bson::Boolean(v) => json!(v),
             Bson::Null => Value::Null,
             Bson::RegularExpression(Regex { pattern, options }) => {
@@ -407,6 +416,8 @@ impl Bson {
             }),
             Bson::Int32(v) => v.into(),
             Bson::Int64(v) => v.into(),
+            Bson::UInt32(v) => v.into(),
+            Bson::UInt64(v) => v.into(),
             Bson::Timestamp(Timestamp { time, increment }) => json!({
                 "$timestamp": {
                     "t": time,
@@ -505,6 +516,8 @@ impl Bson {
             Bson::JavaScriptCodeWithScope(..) => ElementType::JavaScriptCodeWithScope,
             Bson::Int32(..) => ElementType::Int32,
             Bson::Int64(..) => ElementType::Int64,
+            Bson::UInt32(..) => ElementType::UInt32,
+            Bson::UInt64(..) => ElementType::UInt64,
             Bson::Timestamp(..) => ElementType::Timestamp,
             Bson::Binary(..) => ElementType::Binary,
             Bson::ObjectId(..) => ElementType::ObjectId,
@@ -889,6 +902,22 @@ impl Bson {
         }
     }
 
+    /// If `Bson` is `U32`, return its value. Returns `None` otherwise
+    pub fn as_u32(&self) -> Option<u32> {
+        match *self {
+            Bson::UInt32(v) => Some(v),
+            _ => None,
+        }
+    }
+
+    /// If `Bson` is `U64`, return its value. Returns `None` otherwise
+    pub fn as_u64(&self) -> Option<u64> {
+        match *self {
+            Bson::UInt64(v) => Some(v),
+            _ => None,
+        }
+    }
+
     /// If `Bson` is `Objectid`, return its value. Returns `None` otherwise
     pub fn as_object_id(&self) -> Option<oid::ObjectId> {
         match *self {
