@@ -102,6 +102,8 @@ impl Bson {
             Bson::Double(f) => Unexpected::Float(*f),
             Bson::Int32(i) => Unexpected::Signed(*i as i64),
             Bson::Int64(i) => Unexpected::Signed(*i),
+            Bson::UInt32(i) => Unexpected::Unsigned(*i as u64),
+            Bson::UInt64(i) => Unexpected::Unsigned(*i),
             Bson::JavaScriptCode(_) => Unexpected::Other("javascript code"),
             Bson::JavaScriptCodeWithScope(_) => Unexpected::Other("javascript code with scope"),
             Bson::MaxKey => Unexpected::Other("maxkey"),
