@@ -524,39 +524,18 @@ enum BsonInteger {
     Int64(i64),
 }
 
-fn _convert_unsigned<E: Error>(value: u64) -> Result<BsonInteger, E> {
-    if let Ok(int32) = i32::try_from(value) {
-        Ok(BsonInteger::Int32(int32))
-    } else if let Ok(int64) = i64::try_from(value) {
-        Ok(BsonInteger::Int64(int64))
-    } else {
-        Err(Error::custom(format!(
-            "cannot represent {} as a signed number",
-            value
-        )))
-    }
-}
-
 fn convert_unsigned_to_signed<E>(value: u64) -> Result<Bson, E>
 where
     E: Error,
 {
-    let bi = _convert_unsigned(value)?;
-    match bi {
-        BsonInteger::Int32(i) => Ok(Bson::Int32(i)),
-        BsonInteger::Int64(i) => Ok(Bson::Int64(i)),
-    }
+    Ok(Bson::UInt64(value))
 }
 
 pub(crate) fn convert_unsigned_to_signed_raw<'a, E>(value: u64) -> Result<RawBson<'a>, E>
 where
     E: Error,
 {
-    let bi = _convert_unsigned(value)?;
-    match bi {
-        BsonInteger::Int32(i) => Ok(RawBson::Int32(i)),
-        BsonInteger::Int64(i) => Ok(RawBson::Int64(i)),
-    }
+    Ok(RawBson::UInt64(value))
 }
 
 /// Serde Deserializer
