@@ -1,12 +1,12 @@
 //! [BSON Decimal128](https://github.com/mongodb/specifications/blob/master/source/bson-decimal128/decimal128.rst) data type representation
 
-use std::fmt;
+use std::{convert::TryInto, fmt};
 
 /// Struct representing a BSON Decimal128 type.
 ///
 /// Currently, this type can only be used to round-trip through BSON. See
 /// [RUST-36](https://jira.mongodb.org/browse/RUST-36) to track the progress towards a complete implementation.
-#[derive(Clone, PartialEq)]
+#[derive(Copy, Clone, PartialEq)]
 pub struct Decimal128 {
     /// BSON bytes containing the decimal128. Stored for round tripping.
     pub(crate) bytes: [u8; 128 / 8],
@@ -22,6 +22,13 @@ impl Decimal128 {
     pub fn bytes(&self) -> [u8; 128 / 8] {
         self.bytes
     }
+
+    pub(crate) fn deserialize_from_slice<E: serde::de::Error>(
+        bytes: &[u8],
+    ) -> std::result::Result<Self, E> {
+        let arr: [u8; 128 / 8] = bytes.try_into().map_err(E::custom)?;
+        Ok(Decimal128 { bytes: arr })
+    }
 }
 
 impl fmt::Debug for Decimal128 {
