@@ -51,7 +51,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! bson = "2.2.0"
+//! bson = "2.0.0"
 //! ```
 //!
 //! Note that if you are using `bson` through the `mongodb` crate, you do not need to specify it in
@@ -264,7 +264,7 @@
 //! it will only happen in a minor or major version release.
 
 #![allow(clippy::cognitive_complexity)]
-#![doc(html_root_url = "https://docs.rs/bson/2.2.0")]
+#![doc(html_root_url = "https://docs.rs/bson/2.0.0")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
 #[doc(inline)]
@@ -272,18 +272,17 @@ pub use self::{
     bson::{Array, Binary, Bson, DbPointer, Document, JavaScriptCodeWithScope, Regex, Timestamp},
     datetime::DateTime,
     de::{
-        from_bson, from_bson_with_options, from_document, from_document_with_options, from_reader,
-        from_reader_utf8_lossy, from_slice, from_slice_utf8_lossy, Deserializer,
-        DeserializerOptions,
+        from_bson, from_bson_with_options, from_document, from_document_with_options, from_reader, from_reader_utf8_lossy,
+        from_slice, from_slice_utf8_lossy, Deserializer, DeserializerOptions,
     },
     decimal128::Decimal128,
     raw::{
-        RawArray, RawArrayBuf, RawBinaryRef, RawBson, RawBsonRef, RawDbPointerRef, RawDocument,
-        RawDocumentBuf, RawJavaScriptCodeWithScope, RawJavaScriptCodeWithScopeRef, RawRegexRef,
+        RawArray, RawBinary, RawBson, RawDbPointer, RawDocument, RawDocumentBuf, RawJavaScriptCodeWithScope,
+        RawRegex,
     },
     ser::{
-        to_bson, to_bson_with_options, to_document, to_document_with_options, to_raw_document_buf,
-        to_vec, Serializer, SerializerOptions,
+        to_bson, to_bson_with_options, to_document, to_document_with_options, to_vec, Serializer,
+        SerializerOptions,
     },
     uuid::{Uuid, UuidRepresentation},
 };
