@@ -1,25 +1,23 @@
-use std::{borrow::Cow, convert::TryFrom};
+use std::convert::TryFrom;
 
 use serde::{ser::SerializeSeq, Deserialize, Serialize};
 
 use super::{
     error::{ValueAccessError, ValueAccessErrorKind, ValueAccessResult},
-    serde::OwnedOrBorrowedRawArray,
     Error,
     Iter,
-    RawBinaryRef,
-    RawBsonRef,
+    RawBinary,
+    RawBson,
     RawDocument,
-    RawRegexRef,
+    RawRegex,
     Result,
 };
 use crate::{
     oid::ObjectId,
-    raw::RAW_ARRAY_NEWTYPE,
-    spec::ElementType,
+    raw::{RawBsonVisitor, RAW_ARRAY_NEWTYPE},
+    spec::{BinarySubtype, ElementType},
     Bson,
     DateTime,
-    RawArrayBuf,
     Timestamp,
 };
 
@@ -43,7 +41,7 @@ use crate::{
 /// };
 /// let bytes = bson::to_vec(&doc)?;
 ///
-/// let rawdoc = RawDocument::from_bytes(bytes.as_slice())?;
+/// let rawdoc = RawDocument::new(bytes.as_slice())?;
 /// let rawarray = rawdoc.get_array("x")?;
 ///
 /// for v in rawarray {
@@ -66,7 +64,7 @@ use crate::{
 /// };
 /// let bytes = bson::to_vec(&doc)?;
 ///
-/// let rawdoc = RawDocument::from_bytes(bytes.as_slice())?;
+/// let rawdoc = RawDocument::new(bytes.as_slice())?;
 /// let rawarray = rawdoc.get_array("x")?;
 ///
 /// assert_eq!(rawarray.get_bool(1)?, true);
@@ -92,15 +90,8 @@ impl RawArray {
         unsafe { &*(doc as *const RawDocument as *const RawArray) }
     }
 
-    /// Convert this borrowed [`RawArray`] into an owned [`RawArrayBuf`].
-    ///
-    /// This involves a traversal of the array to count the values.
-    pub fn to_raw_array_buf(&self) -> RawArrayBuf {
-        RawArrayBuf::from_raw_document_buf(self.doc.to_raw_document_buf())
-    }
-
     /// Gets a reference to the value at the given index.
-    pub fn get(&self, index: usize) -> Result<Option<RawBsonRef<'_>>> {
+    pub fn get(&self, index: usize) -> Result<Option<RawBson<'_>>> {
         self.into_iter().nth(index).transpose()
     }
 
@@ -108,7 +99,7 @@ impl RawArray {
         &'a self,
         index: usize,
         expected_type: ElementType,
-        f: impl FnOnce(RawBsonRef<'a>) -> Option<T>,
+        f: impl FnOnce(RawBson<'a>) -> Option<T>,
     ) -> ValueAccessResult<T> {
         let bson = self
             .get(index)
@@ -135,88 +126,79 @@ impl RawArray {
     /// Gets the BSON double at the given index or returns an error if the value at that index isn't
     /// a double.
     pub fn get_f64(&self, index: usize) -> ValueAccessResult<f64> {
-        self.get_with(index, ElementType::Double, RawBsonRef::as_f64)
+        self.get_with(index, ElementType::Double, RawBson::as_f64)
     }
 
     /// Gets a reference to the string at the given index or returns an error if the
     /// value at that index isn't a string.
     pub fn get_str(&self, index: usize) -> ValueAccessResult<&str> {
-        self.get_with(index, ElementType::String, RawBsonRef::as_str)
+        self.get_with(index, ElementType::String, RawBson::as_str)
     }
 
     /// Gets a reference to the document at the given index or returns an error if the
     /// value at that index isn't a document.
     pub fn get_document(&self, index: usize) -> ValueAccessResult<&RawDocument> {
-        self.get_with(
-            index,
-            ElementType::EmbeddedDocument,
-            RawBsonRef::as_document,
-        )
+        self.get_with(index, ElementType::EmbeddedDocument, RawBson::as_document)
     }
 
     /// Gets a reference to the array at the given index or returns an error if the
     /// value at that index isn't a array.
     pub fn get_array(&self, index: usize) -> ValueAccessResult<&RawArray> {
-        self.get_with(index, ElementType::Array, RawBsonRef::as_array)
+        self.get_with(index, ElementType::Array, RawBson::as_array)
     }
 
     /// Gets a reference to the BSON binary value at the given index or returns an error if the
     /// value at that index isn't a binary.
-    pub fn get_binary(&self, index: usize) -> ValueAccessResult<RawBinaryRef<'_>> {
-        self.get_with(index, ElementType::Binary, RawBsonRef::as_binary)
+    pub fn get_binary(&self, index: usize) -> ValueAccessResult<RawBinary<'_>> {
+        self.get_with(index, ElementType::Binary, RawBson::as_binary)
     }
 
     /// Gets the ObjectId at the given index or returns an error if the value at that index isn't an
     /// ObjectId.
     pub fn get_object_id(&self, index: usize) -> ValueAccessResult<ObjectId> {
-        self.get_with(index, ElementType::ObjectId, RawBsonRef::as_object_id)
+        self.get_with(index, ElementType::ObjectId, RawBson::as_object_id)
     }
 
     /// Gets the boolean at the given index or returns an error if the value at that index isn't a
     /// boolean.
     pub fn get_bool(&self, index: usize) -> ValueAccessResult<bool> {
-        self.get_with(index, ElementType::Boolean, RawBsonRef::as_bool)
+        self.get_with(index, ElementType::Boolean, RawBson::as_bool)
     }
 
     /// Gets the DateTime at the given index or returns an error if the value at that index isn't a
     /// DateTime.
     pub fn get_datetime(&self, index: usize) -> ValueAccessResult<DateTime> {
-        self.get_with(index, ElementType::DateTime, RawBsonRef::as_datetime)
+        self.get_with(index, ElementType::DateTime, RawBson::as_datetime)
     }
 
     /// Gets a reference to the BSON regex at the given index or returns an error if the
     /// value at that index isn't a regex.
-    pub fn get_regex(&self, index: usize) -> ValueAccessResult<RawRegexRef<'_>> {
-        self.get_with(index, ElementType::RegularExpression, RawBsonRef::as_regex)
+    pub fn get_regex(&self, index: usize) -> ValueAccessResult<RawRegex<'_>> {
+        self.get_with(index, ElementType::RegularExpression, RawBson::as_regex)
     }
 
     /// Gets a reference to the BSON timestamp at the given index or returns an error if the
     /// value at that index isn't a timestamp.
     pub fn get_timestamp(&self, index: usize) -> ValueAccessResult<Timestamp> {
-        self.get_with(index, ElementType::Timestamp, RawBsonRef::as_timestamp)
+        self.get_with(index, ElementType::Timestamp, RawBson::as_timestamp)
     }
 
     /// Gets the BSON int32 at the given index or returns an error if the value at that index isn't
     /// a 32-bit integer.
     pub fn get_i32(&self, index: usize) -> ValueAccessResult<i32> {
-        self.get_with(index, ElementType::Int32, RawBsonRef::as_i32)
+        self.get_with(index, ElementType::Int32, RawBson::as_i32)
     }
 
     /// Gets BSON int64 at the given index or returns an error if the value at that index isn't a
     /// 64-bit integer.
     pub fn get_i64(&self, index: usize) -> ValueAccessResult<i64> {
-        self.get_with(index, ElementType::Int64, RawBsonRef::as_i64)
+        self.get_with(index, ElementType::Int64, RawBson::as_i64)
     }
 
     /// Gets a reference to the raw bytes of the [`RawArray`].
     pub fn as_bytes(&self) -> &[u8] {
         self.doc.as_bytes()
     }
-
-    /// Whether this array contains any elements or not.
-    pub fn is_empty(&self) -> bool {
-        self.doc.is_empty()
-    }
 }
 
 impl std::fmt::Debug for RawArray {
@@ -240,23 +222,9 @@ impl TryFrom<&RawArray> for Vec<Bson> {
     }
 }
 
-impl ToOwned for RawArray {
-    type Owned = RawArrayBuf;
-
-    fn to_owned(&self) -> Self::Owned {
-        self.to_raw_array_buf()
-    }
-}
-
-impl<'a> From<&'a RawArray> for Cow<'a, RawArray> {
-    fn from(rdr: &'a RawArray) -> Self {
-        Cow::Borrowed(rdr)
-    }
-}
-
 impl<'a> IntoIterator for &'a RawArray {
     type IntoIter = RawArrayIter<'a>;
-    type Item = Result<RawBsonRef<'a>>;
+    type Item = Result<RawBson<'a>>;
 
     fn into_iter(self) -> RawArrayIter<'a> {
         RawArrayIter {
@@ -271,9 +239,9 @@ pub struct RawArrayIter<'a> {
 }
 
 impl<'a> Iterator for RawArrayIter<'a> {
-    type Item = Result<RawBsonRef<'a>>;
+    type Item = Result<RawBson<'a>>;
 
-    fn next(&mut self) -> Option<Result<RawBsonRef<'a>>> {
+    fn next(&mut self) -> Option<Result<RawBson<'a>>> {
         match self.inner.next() {
             Some(Ok((_, v))) => Some(Ok(v)),
             Some(Err(e)) => Some(Err(e)),
@@ -287,11 +255,15 @@ impl<'de: 'a, 'a> Deserialize<'de> for &'a RawArray {
     where
         D: serde::Deserializer<'de>,
     {
-        match OwnedOrBorrowedRawArray::deserialize(deserializer)? {
-            OwnedOrBorrowedRawArray::Borrowed(b) => Ok(b),
-            o => Err(serde::de::Error::custom(format!(
-                "expected borrowed raw array, instead got owned {:?}",
-                o
+        match deserializer.deserialize_newtype_struct(RAW_ARRAY_NEWTYPE, RawBsonVisitor)? {
+            RawBson::Array(d) => Ok(d),
+            RawBson::Binary(b) if b.subtype == BinarySubtype::Generic => {
+                let doc = RawDocument::new(b.bytes).map_err(serde::de::Error::custom)?;
+                Ok(RawArray::from_doc(doc))
+            }
+            b => Err(serde::de::Error::custom(format!(
+                "expected raw array reference, instead got {:?}",
+                b
             ))),
         }
     }
