@@ -72,6 +72,22 @@ fn write_i64<W: Write + ?Sized>(writer: &mut W, val: i64) -> Result<()> {
         .map_err(From::from)
 }
 
+#[inline]
+pub(crate) fn write_u32<W: Write + ?Sized>(writer: &mut W, val: u32) -> Result<()> {
+    writer
+        .write_all(&val.to_le_bytes())
+        .map(|_| ())
+        .map_err(From::from)
+}
+
+#[inline]
+fn write_u64<W: Write + ?Sized>(writer: &mut W, val: u64) -> Result<()> {
+    writer
+        .write_all(&val.to_le_bytes())
+        .map(|_| ())
+        .map_err(From::from)
+}
+
 #[inline]
 fn write_f64<W: Write + ?Sized>(writer: &mut W, val: f64) -> Result<()> {
     writer
@@ -162,6 +178,8 @@ pub(crate) fn serialize_bson<W: Write + ?Sized>(
         }
         Bson::Int32(v) => write_i32(writer, v),
         Bson::Int64(v) => write_i64(writer, v),
+        Bson::UInt32(v) => write_u32(writer, v),
+        Bson::UInt64(v) => write_u64(writer, v),
         Bson::Timestamp(ts) => write_i64(writer, ts.to_le_i64()),
         Bson::Binary(Binary { subtype, ref bytes }) => write_binary(writer, bytes, subtype),
         Bson::DateTime(ref v) => write_i64(writer, v.timestamp_millis()),
